@@ -10,7 +10,7 @@ SV *rect( SV *rect, int* new_rect_made)
     if( !SvOK(rect) )
     {
         SDL_Rect* r = safemalloc( sizeof(SDL_Rect) );
-		(*new_rect_made) = 1;
+        (*new_rect_made) = 1;
         r->x        = 0;
         r->y        = 0;
         r->w        = 0;
@@ -20,14 +20,15 @@ SV *rect( SV *rect, int* new_rect_made)
     else if( sv_derived_from(rect, "ARRAY") )
     {
         SDL_Rect* r = safemalloc( sizeof(SDL_Rect) );
-	   		(*new_rect_made) = 1;
+        (*new_rect_made) = 1;
         int ra[4];
-        int i       = 0;
-        AV* recta   = (AV*)SvRV(rect);
+        int i            = 0;
+        AV* recta        = (AV*)SvRV(rect);
+        int len          = av_len(recta);
         for(i = 0; i < 4; i++)
         { 
-            SV* iv = av_shift(recta);
-            if( !SvOK( iv ) || iv == &PL_sv_undef )
+            SV* iv = i > len ? NULL : AvARRAY(recta)[i];
+            if( iv == NULL || !SvOK( iv ) || iv == &PL_sv_undef )
                 ra[i] = 0;
             else
                 ra[i] = SvIV( iv );
@@ -38,13 +39,14 @@ SV *rect( SV *rect, int* new_rect_made)
     }
     else if( sv_isobject(rect) && sv_derived_from(rect, "SDL::Rect") )
     {
-		(*new_rect_made) = 0;
-       	retval = rect;
+        (*new_rect_made) = 0;
+        retval = rect;
+        SvREFCNT_inc(rect);
     }
     else
         croak("Rect must be number or arrayref or SDL::Rect or undef");
 
-	SvREFCNT_inc(rect);
+    //SvREFCNT_inc(rect);
     return retval;
 }
 
@@ -105,19 +107,16 @@ SV *_color_number( SV *color, SV *alpha )
 
 AV *_color_arrayref( AV *color, SV *alpha )
 {
-    AV *retval = newAV();
-		retval = (AV*)sv_2mortal((SV*)retval);
+    AV *retval = (AV*)sv_2mortal((SV*)newAV());
     int length = SvTRUE(alpha) ? 4 : 3;
     int i      = 0;
     for(i = 0; i < length; i++)
     {
-        SV *c = *av_fetch(color, i, 0);
-
-        if( av_len(color) < i || !SvOK(*av_fetch(color, i, 0)) )
+        if( av_len(color) < i || !SvOK(AvARRAY(color)[i]) )
             av_push(retval, newSVuv(i == 3 ? 0xFF : 0));
         else
         {
-            int c = SvIV(*av_fetch(color, i, 0));
+            int c = SvIV(AvARRAY(color)[i]);
             if( c > 0xFF )
             {
                 croak("Number in color arrayref was greater than maximum expected: 0xFF");
@@ -132,116 +131,106 @@ AV *_color_arrayref( AV *color, SV *alpha )
                 av_push(retval, newSVuv(c));
         }
     }
+    
     return retval;
-
 }
 
 
 AV* __list_rgb( SV* color )
 {
-        char *format = _color_format(color);
-        AV* RETVAL       = newAV();
-        if ( 0 == strcmp("number", format) )
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            unsigned int _color = SvUV(_color_number(color, newSVuv(0)));
-            av_push(RETVAL, newSVuv(_color >> 16 & 0xFF));
-            av_push(RETVAL, newSVuv(_color >>  8 & 0xFF));
-            av_push(RETVAL, newSVuv(_color       & 0xFF));
-        }
-        else if ( 0 == strcmp("arrayref", format) )
-        {
-            RETVAL = _color_arrayref((AV *)SvRV(color), sv_2mortal(newSVuv(0)));
-        }
-        else if ( 0 == strcmp("SDLx::Color", format) )
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            SDL_Color *_color = (SDL_Color *)bag2obj(color);
-            av_push(RETVAL, newSVuv(_color->r));
-            av_push(RETVAL, newSVuv(_color->g));
-            av_push(RETVAL, newSVuv(_color->b));
-        }
-        else
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            av_push(RETVAL, newSVuv(0));
-            av_push(RETVAL, newSVuv(0));
-            av_push(RETVAL, newSVuv(0));
-        }
-	return RETVAL;
+    char *format = _color_format(color);
+    AV* RETVAL   = newAV();
+    if ( 0 == strcmp("number", format) )
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        unsigned int _color = SvUV(sv_2mortal(_color_number(color, newSVuv(0))));
+        av_push(RETVAL, newSVuv(_color >> 16 & 0xFF));
+        av_push(RETVAL, newSVuv(_color >>  8 & 0xFF));
+        av_push(RETVAL, newSVuv(_color       & 0xFF));
+    }
+    else if ( 0 == strcmp("arrayref", format) )
+    {
+        RETVAL = _color_arrayref((AV *)SvRV(color), sv_2mortal(newSVuv(0)));
+    }
+    else if ( 0 == strcmp("SDLx::Color", format) )
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        SDL_Color *_color = (SDL_Color *)bag2obj(color);
+        av_push(RETVAL, newSVuv(_color->r));
+        av_push(RETVAL, newSVuv(_color->g));
+        av_push(RETVAL, newSVuv(_color->b));
+    }
+    else
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        av_push(RETVAL, newSVuv(0));
+        av_push(RETVAL, newSVuv(0));
+        av_push(RETVAL, newSVuv(0));
+    }
+    
+    return RETVAL;
 }
 
 
 AV* __list_rgba( SV* color )
 {
+    char *format = _color_format(color);
+    AV* RETVAL   = newAV();
+    if ( 0 == strcmp("number", format) )
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        unsigned int _color = SvUV(sv_2mortal(_color_number(color, sv_2mortal(newSVuv(1)))));
+        av_push(RETVAL, newSVuv(_color >> 24 & 0xFF));
+        av_push(RETVAL, newSVuv(_color >> 16 & 0xFF));
+        av_push(RETVAL, newSVuv(_color >>  8 & 0xFF));
+        av_push(RETVAL, newSVuv(_color       & 0xFF));
+    }
+    else if ( 0 == strcmp("arrayref", format) )
+    {
+        RETVAL = _color_arrayref((AV *)SvRV(color), sv_2mortal(newSVuv(1)));
+    }
+    else if ( 0 == strcmp("SDLx::Color", format) )
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        SDL_Color *_color = (SDL_Color*)bag2obj(color);
+        av_push(RETVAL, newSVuv(_color->r));
+        av_push(RETVAL, newSVuv(_color->g));
+        av_push(RETVAL, newSVuv(_color->b));
+        av_push(RETVAL, newSVuv(0xFF));
+    }
+    else
+    {
+        RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
+        av_push(RETVAL, newSVuv(0));
+        av_push(RETVAL, newSVuv(0));
+        av_push(RETVAL, newSVuv(0));
+        av_push(RETVAL, newSVuv(0xFF));
+    }
 
-        char *format = _color_format(color);
-        AV* RETVAL       = newAV();
-        if ( 0 == strcmp("number", format) )
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            unsigned int _color = SvUV(_color_number(color, sv_2mortal(newSVuv(1))));
-            av_push(RETVAL, newSVuv(_color >> 24 & 0xFF));
-            av_push(RETVAL, newSVuv(_color >> 16 & 0xFF));
-            av_push(RETVAL, newSVuv(_color >>  8 & 0xFF));
-            av_push(RETVAL, newSVuv(_color       & 0xFF));
-        }
-        else if ( 0 == strcmp("arrayref", format) )
-        {
-            RETVAL = _color_arrayref((AV *)SvRV(color), sv_2mortal(newSVuv(1)));
-        }
-        else if ( 0 == strcmp("SDLx::Color", format) )
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            SDL_Color *_color = (SDL_Color*)bag2obj(color);
-            av_push(RETVAL, newSVuv(_color->r));
-            av_push(RETVAL, newSVuv(_color->g));
-            av_push(RETVAL, newSVuv(_color->b));
-            av_push(RETVAL, newSVuv(0xFF));
-        }
-        else
-        {
-            RETVAL = newAV();
-            RETVAL = (AV*)sv_2mortal((SV *)RETVAL);
-            av_push(RETVAL, newSVuv(0));
-            av_push(RETVAL, newSVuv(0));
-            av_push(RETVAL, newSVuv(0));
-            av_push(RETVAL, newSVuv(0xFF));
-        }
-
-	return RETVAL;
-
+    return RETVAL;
 }
 
 
 unsigned int __map_rgb( SV* color, SDL_PixelFormat* format )
 {
-	Uint8 r,b,g;
-	AV* a;
-	a = __list_rgb( color );
+	Uint8 r,g,b;
+	AV* a = __list_rgb( color );
 	r = SvUV(*av_fetch(a, 0, 0));
-	b = SvUV(*av_fetch(a, 1, 0));
-	g = SvUV(*av_fetch(a, 2, 0));
+	g = SvUV(*av_fetch(a, 1, 0));
+	b = SvUV(*av_fetch(a, 2, 0));
 
-	return SDL_MapRGB( format, r,b,g ); 
+	return SDL_MapRGB( format, r, g, b ); 
 
 }
 
 unsigned int __map_rgba( SV* color, SDL_PixelFormat* format )
 {
-	int r,b,g,a;
-	AV* ar;
-	ar = __list_rgba( color );
+	int r,g,b,a;
+	AV* ar = __list_rgba( color );
 	r = SvUV(*av_fetch(ar, 0, 0));
-	b = SvUV(*av_fetch(ar, 1, 0));
-	g = SvUV(*av_fetch(ar, 2, 0));
+	g = SvUV(*av_fetch(ar, 1, 0));
+	b = SvUV(*av_fetch(ar, 2, 0));
 	a = SvUV(*av_fetch(ar, 3, 0));
 
-	return SDL_MapRGBA( format, r,b,g,a ); 
-
+	return SDL_MapRGBA( format, r, g, b, a );
 }
